@@ -16,7 +16,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: alsa_driver.c,v 1.48 2004/01/04 17:55:47 joq Exp $
+    $Id: alsa_driver.c,v 1.51 2004/03/27 22:08:55 joq Exp $
 */
 
 #include <math.h>
@@ -26,20 +26,21 @@
 #include <stdlib.h>
 #include <errno.h>
 #include <stdarg.h>
-#include <getopt.h>
 #include <signal.h>
 #include <sys/types.h>
 #include <regex.h>
 #include <string.h>
-#include "alsa_driver.h"
-#include <jack/types.h>
+
 #include <jack/internal.h>
 #include <jack/engine.h>
+
+#include <sysdeps/time.h>
+
+#include "alsa_driver.h"
 #include "hammerfall.h"
 #include "hdsp.h"
 #include "ice1712.h"
 #include "generic.h"
-#include <jack/time.h>
 
 extern void store_work_time (int);
 extern void store_wait_time (int);
@@ -1874,7 +1875,7 @@ alsa_driver_new (char *name, char *playback_alsa_device,
 		user_capture_nchnls,user_playback_nchnls,
 		hw_monitoring ? "hwmon": "nomon",
 		hw_metering ? "hwmeter":"swmeter",
-		soft_mode ? "soft-mode":"rt",
+		soft_mode ? "soft-mode":"-",
 		shorts_first ? "16bit":"32bit");
 
 	driver = (alsa_driver_t *) calloc (1, sizeof (alsa_driver_t));
