@@ -16,7 +16,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: engine.h,v 1.36 2003/11/02 04:18:06 joq Exp $
+    $Id: engine.h,v 1.41 2004/04/12 04:14:22 pbd Exp $
 */
 
 #ifndef __jack_engine_h__
@@ -112,6 +112,7 @@ struct _jack_engine {
     int		    rtpriority;
     char	    freewheeling;
     char	    verbose;
+    char	    temporary;
     int		    reordered;
     int		    watchdog_check;
     pid_t           wait_pid;
@@ -136,7 +137,7 @@ struct _jack_engine {
     float	    spare_usecs;
     float	    usecs_per_cycle;
     
-#if defined(__APPLE__) && defined(__POWERPC__) 
+#ifdef JACK_USE_MACH_THREADS
     /* specific resources for server/client real-time thread communication */
     mach_port_t servertask, bp;
     int portnum;
@@ -147,8 +148,10 @@ struct _jack_engine {
 /* public functions */
 
 jack_engine_t  *jack_engine_new (int real_time, int real_time_priority,
-                                 int verbose, int client_timeout,
-                                 pid_t waitpid, JSList * drivers);
+				 int do_mlock, int temporary,
+				 int verbose, int client_timeout,
+				 unsigned int port_max,
+                                 pid_t waitpid, JSList *drivers);
 int             jack_engine_delete (jack_engine_t *);
 int             jack_run (jack_engine_t *engine);
 int             jack_wait (jack_engine_t *engine);
