@@ -21,7 +21,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: internal.h,v 1.59 2003/12/16 06:08:38 joq Exp $
+    $Id: internal.h,v 1.64 2004/04/07 02:32:26 pbd Exp $
 */
 
 #ifndef __jack_internal_h__
@@ -31,25 +31,29 @@
 #include <unistd.h>
 #include <limits.h>
 #include <dlfcn.h>
+#include <pthread.h>
 #include <sys/types.h>
 #include <sys/time.h>
-#include <pthread.h>
 
-#if defined(__APPLE__) && defined(__POWERPC__) 
-    #include "mach_port.h"
-#endif
+/* Needed by <sysdeps/time.h> */
+extern void jack_error (const char *fmt, ...);
 
 #include <jack/jack.h>
 #include <jack/types.h>
 #include <jack/port.h>
 #include <jack/transport.h>
-#include <jack/time.h>
+#include <sysdeps/time.h>
+#include <sysdeps/atomicity.h>
+
+#ifdef JACK_USE_MACH_THREADS
+#include <sysdeps/mach_port.h>
+#endif
 
 #ifdef DEBUG_ENABLED
 #define DEBUG(format,args...) \
 	fprintf (stderr, "jack:%5d:%" PRIu64 " %s:%s:%d: " format "\n", getpid(), jack_get_microseconds(), __FILE__, __FUNCTION__, __LINE__ , ## args)
 #else
-#if defined(__APPLE__) && defined(__POWERPC__) 
+#if JACK_CPP_VARARGS_BROKEN
     #define DEBUG(format...)
 #else
     #define DEBUG(format,args...)
@@ -92,7 +96,7 @@ typedef struct {
     jack_position_t	  pending_time;	/* position for next cycle */
     jack_position_t	  request_time;	/* latest requested position */
     jack_unique_t	  prev_request; /* previous request unique ID */
-    volatile uint32_t	  seq_number;	/* unique ID sequence number */
+    volatile _Atomic_word seq_number;	/* unique ID sequence number */
     int8_t		  new_pos;	/* new position this cycle */
     int8_t		  pending_pos;	/* new position request pending */
     jack_nframes_t	  pending_frame; /* pending frame number */
@@ -106,12 +110,13 @@ typedef struct {
     pid_t                 engine_pid;
     jack_nframes_t	  buffer_size;
     int8_t		  real_time;
+    int8_t		  do_mlock;
     int32_t		  client_priority;
     int32_t		  has_capabilities;
     float                 cpu_load;
     uint32_t		  port_max;
     int32_t		  engine_ok;
-    uint32_t		  n_port_types;
+    jack_port_type_id_t	  n_port_types;
     jack_port_type_info_t port_types[JACK_MAX_PORT_TYPES];
     jack_port_shared_t    ports[0];
 
@@ -164,7 +169,7 @@ typedef volatile struct {
     volatile jack_client_id_t id;         /* w: engine r: engine and client */
     volatile jack_nframes_t  nframes;     /* w: engine r: client */
     volatile jack_client_state_t state;   /* w: engine and client r: engine */
-    volatile int8_t     name[JACK_CLIENT_NAME_SIZE+1];
+    volatile int8_t     name[JACK_CLIENT_NAME_SIZE];
     volatile ClientType type;             /* w: engine r: engine and client */
     volatile int8_t     active;           /* w: engine r: engine and client */
     volatile int8_t     dead;             /* r/w: engine */
@@ -189,6 +194,8 @@ typedef volatile struct {
      */
     JackProcessCallback process;
     void *process_arg;
+    JackThreadInitCallback thread_init;
+    void *thread_init_arg;
     JackBufferSizeCallback bufsize;
     void *bufsize_arg;
     JackSampleRateCallback srate;
@@ -221,7 +228,7 @@ typedef struct {
     int32_t    load;
     ClientType type;
 
-    char name[JACK_CLIENT_NAME_SIZE+1];
+    char name[JACK_CLIENT_NAME_SIZE];
     char object_path[PATH_MAX+1];
     char object_data[1024];
 
@@ -248,7 +255,7 @@ typedef struct {
     jack_client_control_t *client_control; /* JOQ: 64/32 problem */
     jack_control_t        *engine_control; /* JOQ: 64/32 problem */
 
-#if defined(__APPLE__) && defined(__POWERPC__) 
+#ifdef JACK_USE_MACH_THREADS
     /* specific resources for server/client real-time thread communication */
     int32_t	portnum;
 #endif
@@ -289,16 +296,16 @@ struct _jack_request {
     RequestType type;
     union {
 	struct {
-	    char name[JACK_PORT_NAME_SIZE+1];
-	    char type[JACK_PORT_TYPE_SIZE+1];
+	    char name[JACK_PORT_NAME_SIZE];
+	    char type[JACK_PORT_TYPE_SIZE];
 	    uint32_t         flags;
 	    jack_shmsize_t   buffer_size;
 	    jack_port_id_t   port_id;
 	    jack_client_id_t client_id;
 	} port_info;
 	struct {
-	    char source_port[JACK_PORT_NAME_SIZE+1];
-	    char destination_port[JACK_PORT_NAME_SIZE+1];
+	    char source_port[JACK_PORT_NAME_SIZE];
+	    char destination_port[JACK_PORT_NAME_SIZE];
 	} connect;
 	struct {
 	    int32_t nports;
@@ -337,13 +344,13 @@ typedef struct _jack_client_internal {
     void    (*finish)(void *);		/* internal clients only */
     int      error;
     
-#if defined(__APPLE__) && defined(__POWERPC__) 
+#ifdef JACK_USE_MACH_THREADS
     /* specific resources for server/client real-time thread communication */
     mach_port_t serverport;
     trivial_message message;
     int running;
     int portnum;
-#endif
+#endif /* JACK_USE_MACH_THREADS */
     
 } jack_client_internal_t;
 
@@ -363,8 +370,6 @@ extern char *jack_server_dir;
 
 extern void *jack_zero_filled_buffer;
 
-extern void jack_error (const char *fmt, ...);
-
 extern jack_port_functions_t jack_builtin_audio_functions;
 
 extern jack_port_type_info_t jack_builtin_port_types[];
