@@ -16,9 +16,11 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: jackd.c,v 1.29 2003/12/01 19:45:17 pbd Exp $
+    $Id: jackd.c,v 1.36 2004/04/14 16:38:26 joq Exp $
 */
 
+#include <config.h>
+
 #include <stdio.h>
 #include <signal.h>
 #include <getopt.h>
@@ -31,8 +33,6 @@
 #include <dirent.h>
 #include <dlfcn.h>
 
-#include <config.h>
-
 #include <jack/engine.h>
 #include <jack/internal.h>
 #include <jack/driver.h>
@@ -49,15 +49,18 @@
 
 static struct stat pipe_stat;
 
-#endif
+#endif /* USE_CAPABILITIES */
 
 static JSList * drivers = NULL;
 static sigset_t signals;
 static jack_engine_t *engine = 0;
 static int realtime = 0;
 static int realtime_priority = 10;
+static int do_mlock = 1;
+static int temporary = 0;
 static int verbose = 0;
 static int client_timeout = 500; /* msecs */
+static unsigned int port_max = 128;
 
 static void 
 do_nothing_handler (int sig)
@@ -132,8 +135,8 @@ jack_main (jack_driver_desc_t * driver_desc, JSList * driver_params)
 	
 	/* get the engine/driver started */
 
-	if ((engine = jack_engine_new (realtime, realtime_priority,
-				       verbose, client_timeout,
+	if ((engine = jack_engine_new (realtime, realtime_priority, do_mlock,
+				       temporary, verbose, client_timeout, port_max,
 				       getpid(), drivers)) == 0) {
 		fprintf (stderr, "cannot create engine\n");
 		return -1;
@@ -142,7 +145,8 @@ jack_main (jack_driver_desc_t * driver_desc, JSList * driver_params)
 	fprintf (stderr, "loading driver ..\n");
 	
 	if (jack_engine_load_driver (engine, driver_desc, driver_params)) {
-		fprintf (stderr, "cannot load driver module %s\n", driver_desc->name);
+		fprintf (stderr, "cannot load driver module %s\n",
+			 driver_desc->name);
 		return -1;
 	}
 
@@ -253,7 +257,6 @@ jack_drivers_get_descriptor (JSList * drivers, const char * sofile)
 		jack_error ("error closing driver .so '%s': %s\n", filename, dlerror ());
 	}
 
-
 	/* check it doesn't exist already */
 	for (node = drivers; node; node = jack_slist_next (node)) {
 		other_descriptor = (jack_driver_desc_t *) node->data;
@@ -341,12 +344,14 @@ static void usage (FILE *file)
 	copyright (file);
 	fprintf (file, "\n"
 "usage: jackd [ --realtime OR -R [ --realtime-priority OR -P priority ] ]\n"
+"             [ --no-mlock OR -m ]\n"
 "             [ --timeout OR -t client-timeout-in-msecs ]\n"
+"             [ --port-max OR -p maximum-number-of-ports]\n"
 "             [ --verbose OR -v ]\n"
 "             [ --silent OR -s ]\n"
 "             [ --version OR -V ]\n"
 "         -d driver [ ... driver args ... ]\n"
-"             driver can be `alsa', `dummy' or `portaudio'\n\n"
+"             driver can be `alsa', `dummy', `oss' or `portaudio'\n\n"
 "       jackd -d driver --help\n"
 "             to display options for each driver\n\n");
 }	
@@ -374,22 +379,25 @@ int
 main (int argc, char *argv[])
 
 {
-        jack_driver_desc_t * desc;
-	const char *options = "-ad:P:vshVRFl:t:";
+	jack_driver_desc_t * desc;
+	const char *options = "-ad:P:vshVRTFl:t:mp:";
 	struct option long_options[] = 
 	{ 
 		{ "driver", 1, 0, 'd' },
 		{ "verbose", 0, 0, 'v' },
 		{ "help", 0, 0, 'h' },
+		{ "port-max", 1, 0, 'p' },
+		{ "no-mlock", 0, 0, 'm' },
 		{ "realtime", 0, 0, 'R' },
 		{ "realtime-priority", 1, 0, 'P' },
 		{ "timeout", 1, 0, 't' },
+		{ "temporary", 0, 0, 'T' },
 		{ "version", 0, 0, 'V' },
 		{ "silent", 0, 0, 's' },
 		{ 0, 0, 0, 0 }
 	};
-	int option_index;
-	int opt;
+	int opt = 0;
+	int option_index = 0;
 	int seen_driver = 0;
 	char *driver_name = 0;
 	char **driver_args = 0;
@@ -435,11 +443,12 @@ main (int argc, char *argv[])
 			}
 		}
 	}
-#endif
+#endif /* USE_CAPABILITIES */
+
 	opterr = 0;
-	while (!seen_driver && (opt = getopt_long (argc, argv, options,
-						   long_options,
-						   &option_index)) != EOF) {
+	while (!seen_driver &&
+	       (opt = getopt_long (argc, argv, options,
+				   long_options, &option_index)) != EOF) {
 		switch (opt) {
 
 		case 'd':
@@ -455,6 +464,14 @@ main (int argc, char *argv[])
 			jack_set_error_function (silent_jack_error_callback);
 			break;
 
+		case 'm':
+			do_mlock = 0;
+			break;
+
+		case 'p':
+			port_max = (unsigned int) atol (optarg);
+			break;
+
 		case 'P':
 			realtime_priority = atoi (optarg);
 			break;
@@ -463,6 +480,10 @@ main (int argc, char *argv[])
 			realtime = 1;
 			break;
 
+		case 'T':
+			temporary = 1;
+			break;
+
 		case 't':
 			client_timeout = atoi (optarg);
 			break;
