@@ -16,53 +16,53 @@
     along with this program; if not, write to the Free Software 
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: client.c,v 1.62 2004/01/09 18:43:47 joq Exp $
+    $Id: client.c,v 1.80 2004/04/14 03:38:32 trutkin Exp $
 */
 
-#if defined(__APPLE__) && defined(__POWERPC__) 
-    #include "pThreadUtilities.h"
-    #include "ipc.h"
-    #include "fakepoll.h"
-#else
-    #include <sys/poll.h>
-#endif
-
-#include <sys/socket.h>
-#include <sys/un.h>
-#include <pthread.h>
-#include <errno.h>
-#include <fcntl.h>
-#include <sys/types.h>
-#include <sys/ipc.h>
-#include <sys/mman.h>
-
-#include <stdarg.h>
-#include <stdio.h>
-#include <stdint.h>
-#include <regex.h>
-
 #include <config.h>
 
-#include <jack/jack.h>
+#include <pthread.h>
+#include <errno.h>
+#include <fcntl.h>
+#include <stdarg.h>
+#include <stdio.h>
+#ifdef HAVE_STDINT_H
+#include <stdint.h>
+#endif
+#include <regex.h>
+#include <string.h>
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <sys/un.h>
+#ifdef USE_MLOCK
+#include <sys/mman.h>
+#endif /* USE_MLOCK */
+
 #include <jack/internal.h>
+#include <jack/jack.h>
 #include <jack/engine.h>
 #include <jack/pool.h>
-#include <jack/time.h>
 #include <jack/jslist.h>
 #include <jack/version.h>
 #include <jack/shm.h>
 
+#include <sysdeps/time.h>
+JACK_TIME_GLOBAL_DECL;			/* One instance per process. */
+
 #include "local.h"
 
+#include <sysdeps/poll.h>
+#include <sysdeps/ipc.h>
+
+#ifdef JACK_USE_MACH_THREADS
+#include <sysdeps/pThreadUtilities.h>
+#endif
+
 #ifdef WITH_TIMESTAMPS
 #include <jack/timestamps.h>
 #endif /* WITH_TIMESTAMPS */
 
-#ifdef DEFAULT_TMP_DIR
 char *jack_server_dir = DEFAULT_TMP_DIR;
-#else
-char *jack_server_dir = "/tmp";
-#endif
 
 void
 jack_set_server_dir (const char *path)
@@ -113,17 +113,30 @@ void (*jack_error_callback)(const char *desc) = &default_jack_error_callback;
 static int
 oop_client_deliver_request (void *ptr, jack_request_t *req)
 {
+	int wok, rok;
 	jack_client_t *client = (jack_client_t*) ptr;
 
-	if (write (client->request_fd, req, sizeof (*req)) != sizeof (*req)) {
-		jack_error ("cannot send request type %d to server", req->type);
-		req->status = -1;
-	}
-	if (read (client->request_fd, req, sizeof (*req)) != sizeof (*req)) {
-		jack_error ("cannot read result for request type %d from server (%s)", req->type, strerror (errno));
-		req->status = -1;
-	}
+	wok = (write (client->request_fd, req, sizeof (*req))
+	       == sizeof (*req));
+	rok = (read (client->request_fd, req, sizeof (*req))
+	       == sizeof (*req));
 
+	if (wok && rok)			/* everything OK? */
+		return req->status;
+
+	req->status = -1;		/* request failed */
+
+	/* check for server shutdown */
+	if (client->engine->engine_ok == 0)
+		return req->status;
+
+	/* otherwise report errors */
+	if (!wok)
+		jack_error ("cannot send request type %d to server",
+				    req->type);
+	if (!rok)
+		jack_error ("cannot read result for request type %d from"
+			    " server (%s)", req->type, strerror (errno));
 	return req->status;
 }
 
@@ -154,6 +167,9 @@ jack_client_alloc ()
 	client->engine = NULL;
 	client->control = NULL;
 	client->thread_ok = FALSE;
+#if JACK_USE_MACH_THREADS 
+	client->rt_thread_ok = FALSE;
+#endif
 	client->first_active = TRUE;
 	client->on_shutdown = NULL;
 	client->n_port_types = 0;
@@ -162,6 +178,9 @@ jack_client_alloc ()
 	return client;
 }
 
+/*
+ * Build the jack_client_t structure for an internal client.
+ */
 jack_client_t *
 jack_client_alloc_internal (jack_client_control_t *cc, jack_engine_t* engine)
 {
@@ -379,9 +398,134 @@ server_event_connect (jack_client_t *client)
 	return fd;
 }
 
+/* Exec the JACK server in this process.  Does not return. */
+static void
+_start_server (void)
+{
+	FILE* fp = 0;
+	char filename[255];
+	char arguments[255];
+	char buffer[255];
+	char* command = 0;
+	size_t pos = 0;
+	size_t result = 0;
+	char** argv = 0;
+	int i = 0;
+	int good = 0;
+	int ret;
+
+
+	snprintf(filename, 255, "%s/.jackdrc", getenv("HOME"));
+	fp = fopen(filename, "r");
+
+	if (!fp) {
+		fp = fopen("/etc/jackd.conf", "r");
+	}
+
+	if (fp) {
+		arguments[0] = '\0';
+		ret = fscanf(fp, "%s", buffer);
+		while(ret != 0 && ret != EOF) {
+			strcat(arguments, buffer);
+			strcat(arguments, " ");
+			ret = fscanf(fp, "%s", buffer);
+		}
+		if (strlen(arguments) > 0) {
+			good = 1;
+		}
+	}
+
+	if (!good) {
+#if defined(USE_CAPABILITIES)
+		command = JACK_LOCATION "/jackstart";
+		strncpy(arguments, JACK_LOCATION "/jackstart -T -R -d "
+			JACK_DEFAULT_DRIVER " -p 512", 255);
+#else /* !USE_CAPABILITIES */
+		command = JACK_LOCATION "/jackd";
+		strncpy(arguments, JACK_LOCATION "/jackd -T -d "
+			JACK_DEFAULT_DRIVER, 255);
+#endif /* USE_CAPABILITIES */
+	} else {
+		result = strcspn(arguments, " ");
+		command = (char*)malloc(result+1);
+		strncpy(command, arguments, result);
+		command[result] = '\0';
+	}
+
+	argv = (char**)malloc(255);
+  
+	while(1) {
+		/* insert -T into arguments */
+		if (i == 1) {
+			argv[i] = (char*)malloc(3);
+			strncpy(argv[i], "-T", 2); 
+			argv[i][2] = '\0';
+			++i;
+		}
+
+		result = strcspn(arguments+pos, " ");
+		if (result == 0) {
+			break;
+		}
+		argv[i] = (char*)malloc(result+1);
+
+		strncpy(argv[i], arguments+pos, result);
+		argv[i][result] = '\0';
+
+		pos += result+1;
+		++i;
+	}
+	argv[i] = 0;
+
+	execv (command, argv);
+
+	/* If execv() succeeds, it does not return.  There's no point
+	 * in calling jack_error() here in the child process. */
+	perror ("exec of JACK server failed");
+}
+
+int
+start_server (void)
+{
+	/* Only fork() a server when $JACK_START_SERVER is defined and
+	 * $JACK_NO_START_SERVER is not. */
+	if (getenv("JACK_START_SERVER") == NULL ||
+	    getenv("JACK_NO_START_SERVER") != NULL) {
+		return 1;
+	}
+
+	/* The double fork() forces the server to become a child of
+	 * init, which will always clean up zombie process state on
+	 * termination.  This even works in strange corner cases where
+	 * the server terminates but this client does not.
+	 *
+	 * Since fork() is usually implemented using copy-on-write
+	 * virtual memory tricks, the overhead of the second fork() is
+	 * probably relatively small.
+	 */
+	switch (fork()) {
+	case 0:				/* child process */
+		switch (fork()) {
+		case 0:			/* grandchild process */
+			_start_server();
+			_exit (99);	/* exec failed */
+		case -1:
+			_exit (98);
+		default:
+			_exit (0);
+		}
+	case -1:			/* fork() error */
+		return 1;		/* failed to start server */
+	}
+
+	/* only the original parent process goes here */
+	return 0;			/* (probably) successful */
+}
+
 static int
-jack_request_client (ClientType type, const char* client_name, const char* so_name, 
-		     const char* so_data, jack_client_connect_result_t *res, int *req_fd)
+jack_request_client (ClientType type, const char* client_name,
+		     const char* so_name, const char* so_data,
+		     jack_client_connect_result_t *res, int *req_fd)
 {
 	jack_client_connect_request_t req;
 
@@ -389,11 +533,11 @@ jack_request_client (ClientType type, const char* client_name, const char* so_na
 
 	memset (&req, 0, sizeof (req));
 
-	if (strlen (client_name) > sizeof (req.name) - 1) {
+	if (strlen (client_name) >= sizeof (req.name)) {
 		jack_error ("\"%s\" is too long to be used as a JACK client"
 			    " name.\n"
 			    "Please use %lu characters or less.",
-			    client_name, sizeof (req.name) - 1);
+			    client_name, sizeof (req.name));
 		return -1;
 	}
 
@@ -414,7 +558,17 @@ jack_request_client (ClientType type, const char* client_name, const char* so_na
 	}
 
 	if ((*req_fd = server_connect (0)) < 0) {
-		goto fail;
+		int trys;
+		if (start_server()) {
+			goto fail;
+		}
+		trys = 5;
+		do {
+			sleep(1);
+			if (--trys < 0) {
+				goto fail;
+			}
+		} while ((*req_fd = server_connect (0)) < 0);
 	}
 
 	req.load = TRUE;
@@ -497,7 +651,7 @@ jack_attach_port_segment (jack_client_t *client, jack_port_type_id_t ptid)
 		client->port_segment = (jack_shm_info_t*)
 			realloc (client->port_segment,
 				 sizeof (jack_shm_info_t) * (ptid+1));
-		
+
 		memset (&client->port_segment[client->n_port_types],
 			0,
 			sizeof (jack_shm_info_t) * 
@@ -509,7 +663,14 @@ jack_attach_port_segment (jack_client_t *client, jack_port_type_id_t ptid)
 
 		/* release any previous segment */
 		
+#if JACK_USE_MACH_THREADS 
+		/* Stephane Letz : letz@grame.fr
+		Need a fix : this crash on MacOSX : temporary removed 
 		jack_release_shm (&client->port_segment[ptid]);
+		*/
+#else
+		jack_release_shm (&client->port_segment[ptid]);
+#endif
 	}
 
 	/* get the index into the shm registry */
@@ -552,7 +713,11 @@ jack_client_new (const char *client_name)
 	   will use the setup in the server's address space.
 	*/
 	jack_init_time ();
-	jack_initialize_shm ();
+
+	if (jack_initialize_shm ()) {
+		jack_error ("Unable to initialize shared memory.");
+		return NULL;
+	}
 
 	if (jack_request_client (ClientExternal, client_name, "", "",
 				 &res, &req_fd)) {
@@ -616,7 +781,7 @@ jack_client_new (const char *client_name)
 
 	client->event_fd = ev_fd;
         
-#if defined(__APPLE__) && defined(__POWERPC__) 
+#ifdef JACK_USE_MACH_THREADS
         /* specific resources for server/client real-time thread
 	 * communication */
 	client->clienttask = mach_task_self();
@@ -630,7 +795,7 @@ jack_client_new (const char *client_name)
             jack_error("Can't allocate mach port");
             goto fail; 
         }; 
-#endif
+#endif /* JACK_USE_MACH_THREADS */
  	return client;
 	
   fail:
@@ -684,6 +849,24 @@ jack_internal_client_close (const char *client_name)
 	return;
 }
 
+#if JACK_USE_MACH_THREADS 
+
+int
+jack_drop_real_time_scheduling (pthread_t thread)
+{
+	setThreadToPriority(thread, 31, false, 10000000);
+	return 0;       
+}
+
+int
+jack_acquire_real_time_scheduling (pthread_t thread, int priority) //priority is unused
+{
+	setThreadToPriority(thread, 96, true, 10000000);
+	return 0;
+}
+
+#else
+
 int
 jack_drop_real_time_scheduling (pthread_t thread)
 {
@@ -718,6 +901,8 @@ jack_acquire_real_time_scheduling (pthread_t thread, int priority)
         return 0;
 }
 
+#endif
+
 int
 jack_set_freewheel (jack_client_t* client, int onoff)
 {
@@ -732,7 +917,11 @@ jack_start_freewheel (jack_client_t* client)
 	jack_client_control_t *control = client->control;
 
 	if (client->engine->real_time) {
+#if JACK_USE_MACH_THREADS 
+		jack_drop_real_time_scheduling (client->process_thread);
+#else
 		jack_drop_real_time_scheduling (client->thread);
+#endif
 	}
 
 	if (control->freewheel_cb) {
@@ -750,8 +939,13 @@ jack_stop_freewheel (jack_client_t* client)
 	}
 
 	if (client->engine->real_time) {
+#if JACK_USE_MACH_THREADS 
+		jack_acquire_real_time_scheduling (client->process_thread,
+						   client->engine->client_priority);
+#else
 		jack_acquire_real_time_scheduling (client->thread,
 						   client->engine->client_priority);
+#endif
 	}
 }
 
@@ -778,15 +972,20 @@ jack_client_thread (void *arg)
 
 	DEBUG ("client thread is now running");
 
-	while (err == 0) {
-	        if (client->engine->engine_ok == 0) {
-		     jack_error ("engine unexpectedly shutdown; "
-				 "thread exiting\n");
-		     if (client->on_shutdown) {
-			     client->on_shutdown (client->on_shutdown_arg);
-		     }
-		     pthread_exit (0);
+	if (client->control->thread_init) {
+		DEBUG ("calling client thread init callback");
+		client->control->thread_init (client->control->thread_init_arg);
+	}
 
+	while (err == 0) {
+
+	        if (client->engine->engine_ok == 0) {
+		     if (client->on_shutdown)
+			     client->on_shutdown (client->on_shutdown_arg);
+		     else
+			     jack_error ("engine unexpectedly shutdown; "
+					 "thread exiting\n");
+		     pthread_exit (0);
 		}
 
 		DEBUG ("client polling on event_fd and graph_wait_fd...");
@@ -1033,7 +1232,7 @@ jack_client_thread (void *arg)
 }
 
 
-#if defined(__APPLE__) && defined(__POWERPC__) 
+#ifdef JACK_USE_MACH_THREADS
 /* real-time thread : separated from the normal client thread, it will
  * communicate with the server using fast mach RPC mechanism */
 
@@ -1045,14 +1244,17 @@ jack_client_process_thread (void *arg)
 	int err = 0;
       
 	client->control->pid = getpid();
-        DEBUG ("client process thread is now running");
+	DEBUG ("client process thread is now running");
         
-        pthread_setcanceltype (PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
+	pthread_setcanceltype (PTHREAD_CANCEL_ASYNCHRONOUS, NULL);
+        
+	client->rt_thread_ok = TRUE;
             
    	while (err == 0) {
         
                 if (jack_client_suspend(client) < 0) {
-                        pthread_exit (0);
+                        jack_error ("jack_client_process_thread : resume error");
+                        goto zombie;
                 }
                 
                 control->awake_at = jack_get_microseconds();
@@ -1061,6 +1263,9 @@ jack_client_process_thread (void *arg)
                  
                 control->state = Running;
 
+		if (control->sync_cb)
+			jack_call_sync_client (client);
+
                 if (control->process) {
                         if (control->process (control->nframes,
 					      control->process_arg) == 0) {
@@ -1069,6 +1274,9 @@ jack_client_process_thread (void *arg)
                 } else {
                         control->state = Finished;
                 }
+
+		if (control->timebase_cb)
+			jack_call_timebase_master (client);
                 
                 control->finished_at = jack_get_microseconds();
                 
@@ -1095,29 +1303,31 @@ jack_client_process_thread (void *arg)
         
         jack_error ("jack_client_process_thread : zombified");
         
+        client->rt_thread_ok = FALSE;
+
 	if (client->on_shutdown) {
 		jack_error ("zombified - calling shutdown handler");
 		client->on_shutdown (client->on_shutdown_arg);
 	} else {
 		jack_error ("zombified - exiting from JACK");
-                jack_client_close (client); /* Need a fix : possibly make client crash if zombified without shutdown handler */
+		jack_client_close (client); /* Need a fix : possibly make client crash if zombified without shutdown handler */
 	}
 
 	pthread_exit (0);
 	/*NOTREACHED*/
 	return 0;
 }
-#endif
+#endif /* JACK_USE_MACH_THREADS */
 
 static int
 jack_start_thread (jack_client_t *client)
 
 {
-	pthread_attr_t *attributes = 0;
-#ifdef USE_CAPABILITIES
+#ifndef JACK_USE_MACH_THREADS
 	int policy = SCHED_OTHER;
 	struct sched_param client_param, temp_param;
 #endif
+	pthread_attr_t *attributes = 0;
 
 	if (client->engine->real_time) {
 		
@@ -1153,35 +1363,26 @@ jack_start_thread (jack_client_t *client)
 			return -1;
 		}
                 
-#if defined(__APPLE__) && defined(__POWERPC__) 
-                // To be implemented
-#else
-                if (mlockall (MCL_CURRENT | MCL_FUTURE) != 0) {
+#ifdef USE_MLOCK
+                if (client->engine->do_mlock
+		    && (mlockall (MCL_CURRENT | MCL_FUTURE) != 0)) {
                     jack_error ("cannot lock down memory for RT thread (%s)",
 				strerror (errno));
 #ifdef ENSURE_MLOCK
                     return -1;
 #endif /* ENSURE_MLOCK */
                 }
-#endif
-        
+#endif /* USE_MLOCK */
 	}
 
+#ifdef JACK_USE_MACH_THREADS
+
 	if (pthread_create (&client->thread, attributes,
 			    jack_client_thread, client)) {
-#ifdef USE_CAPABILITIES
-		if (client->engine->real_time) {
-			/* we are probably dealing with a broken glibc so try
-			   to work around the bug, see below for more details
-			*/
-			goto capabilities_workaround;
-		}
-#endif
 		return -1;
 	}
-        
-#if defined(__APPLE__) && defined(__POWERPC__) 
-        /* a spcial real-time thread to call the "process"
+
+        /* a special real-time thread to call the "process"
 	 * callback. It will communicate with the server using fast
 	 * mach RPC mechanism */
         if (pthread_create (&client->process_thread, attributes,
@@ -1191,20 +1392,22 @@ jack_start_thread (jack_client_t *client)
         }
         if (client->engine->real_time){
             /* time constraint thread */
-            setThreadToPriority(client->process_thread, 96, true, 10000000);
+            setThreadToPriority(client->process_thread, 96, TRUE, 10000000);
         }else{
             /* fixed priority thread */
-            setThreadToPriority(client->process_thread, 63, true, 10000000);
+            setThreadToPriority(client->process_thread, 63, TRUE, 10000000);
         }
-#endif
-            
-	return 0;
 
-#ifdef USE_CAPABILITIES
+#else /* !JACK_USE_MACH_THREADS */
 
-	/* we get here only with engine running realtime and capabilities */
+	if (pthread_create (&client->thread, attributes,
+			    jack_client_thread, client) == 0) {
+		return 0;
+	}
 
- capabilities_workaround:
+	if (!client->engine->real_time) {
+		return -1;
+	}
 
 	/* the version of glibc I've played with has a bug that makes
 	   that code fail when running under a non-root user but with the
@@ -1294,8 +1497,9 @@ jack_start_thread (jack_client_t *client)
 			}
 		}
 	}
+#endif /* JACK_USE_MACH_THREADS */
+
 	return 0;
-#endif
 }
 
 int 
@@ -1308,22 +1512,13 @@ jack_activate (jack_client_t *client)
 	 * usage in jack_start_thread())
 	 */
          
-#if defined(__APPLE__) && defined(__POWERPC__)
-/* a bigger stack makes the application crash... */
-#define BIG_ENOUGH_STACK 10000
-#else
-#define BIG_ENOUGH_STACK 1048576
-#endif
-
-	char buf[BIG_ENOUGH_STACK];
+	char buf[JACK_THREAD_STACK_TOUCH];
 	int i;
 
-	for (i = 0; i < BIG_ENOUGH_STACK; i++) {
+	for (i = 0; i < JACK_THREAD_STACK_TOUCH; i++) {
 		buf[i] = (char) (i & 0xff);
 	}
 
-#undef BIG_ENOUGH_STACK
-
 	if (client->control->type == ClientInternal ||
 	    client->control->type == ClientDriver) {
 		goto startit;
@@ -1365,7 +1560,7 @@ jack_activate (jack_client_t *client)
 			*/
 		}
 	}
-#endif
+#endif /* USE_CAPABILITIES */
 
 	if (client->first_active) {
 
@@ -1421,6 +1616,16 @@ jack_client_close (jack_client_t *client)
 	}
 
 	if (client->control->type == ClientExternal) {
+
+#if JACK_USE_MACH_THREADS 
+		if (client->rt_thread_ok) {
+			// MacOSX pthread_cancel not implemented in
+			// Darwin 5.5, 6.4
+			mach_port_t machThread =
+				pthread_mach_thread_np (client->process_thread);
+			thread_terminate (machThread);
+		}
+#endif
 	
 		/* stop the thread that communicates with the jack
 		 * server, only if it was actually running 
@@ -1599,6 +1804,20 @@ jack_set_process_callback (jack_client_t *client,
 	return 0;
 }
 
+int
+jack_set_thread_init_callback (jack_client_t *client,
+			       JackThreadInitCallback callback, void *arg)
+
+{
+	if (client->control->active) {
+		jack_error ("You cannot set callbacks on an active client.");
+		return -1;
+	}
+	client->control->thread_init_arg = arg;
+	client->control->thread_init = callback;
+	return 0;
+}
+
 int
 jack_set_freewheel_callback (jack_client_t *client,
 			     JackFreewheelCallback callback, void *arg)
@@ -1653,7 +1872,6 @@ jack_get_process_done_fd (jack_client_t *client)
 	return client->graph_next_fd;
 }
 
-
 void
 jack_on_shutdown (jack_client_t *client, void (*function)(void *arg), void *arg)
 {
@@ -1748,68 +1966,20 @@ jack_client_thread_id (jack_client_t *client)
 	return client->thread_id;
 }
 
-#if defined(__APPLE__) && defined(__POWERPC__) 
-
-double __jack_time_ratio;
-
-void jack_init_time ()
+int
+jack_client_name_size(void)
 {
-        mach_timebase_info_data_t info; 
-        mach_timebase_info(&info);
-        __jack_time_ratio = ((float)info.numer/info.denom) / 1000;
-}
-#else
-
-jack_time_t
-jack_get_mhz (void)
-{
-	FILE *f = fopen("/proc/cpuinfo", "r");
-	if (f == 0)
-	{
-		perror("can't open /proc/cpuinfo\n");
-		exit(1);
-	}
-
-	for ( ; ; )
-	{
-		jack_time_t mhz;
-		int ret;
-		char buf[1000];
-
-		if (fgets(buf, sizeof(buf), f) == NULL) {
-			jack_error ("FATAL: cannot locate cpu MHz in "
-				    "/proc/cpuinfo\n");
-			exit(1);
-		}
-
-#if defined(__powerpc__)
-		ret = sscanf(buf, "clock\t: %" SCNu64 "MHz", &mhz);
-#elif defined( __i386__ ) || defined (__hppa__)  || defined (__ia64__) || \
-      defined(__x86_64__)
-		ret = sscanf(buf, "cpu MHz         : %" SCNu64, &mhz);
-#elif defined( __sparc__ )
-		ret = sscanf(buf, "Cpu0Bogo        : %" SCNu64, &mhz);
-#elif defined( __mc68000__ )
-		ret = sscanf(buf, "Clocking:       %" SCNu64, &mhz);
-#elif defined( __s390__  )
-		ret = sscanf(buf, "bogomips per cpu: %" SCNu64, &mhz);
-#else /* MIPS, ARM, alpha */
-		ret = sscanf(buf, "BogoMIPS        : %" SCNu64, &mhz);
-#endif 
-
-		if (ret == 1)
-		{
-			fclose(f);
-			return (jack_time_t)mhz;
-		}
-	}
+	return JACK_CLIENT_NAME_SIZE;
 }
 
-jack_time_t __jack_cpu_mhz;
-
-void jack_init_time ()
+int
+jack_port_name_size(void)
 {
-	__jack_cpu_mhz = jack_get_mhz ();
+	return JACK_PORT_NAME_SIZE;
 }
 
-#endif
+int
+jack_port_type_size(void)
+{
+	return JACK_PORT_TYPE_SIZE;
+}
