@@ -16,23 +16,25 @@
     along with this program; if not, write to the Free Software 
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
     
-    $Id: driver.c,v 1.13 2003/12/10 06:52:17 joq Exp $
+    $Id: driver.c,v 1.16 2004/04/07 04:52:59 joq Exp $
 */
 
+#include <config.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdarg.h>
 #include <stdlib.h>
 #include <stdio.h>
-#include <sys/mman.h>
 #include <errno.h>
 
-#include <config.h>
-
-#include <jack/driver.h>
 #include <jack/internal.h>
+#include <jack/driver.h>
 #include <jack/engine.h>
 
+#ifdef USE_MLOCK
+#include <sys/mman.h>
+#endif /* USE_MLOCK */
+
 static int dummy_attach (jack_driver_t *drv, jack_engine_t *eng) { return 0; }
 static int dummy_detach (jack_driver_t *drv, jack_engine_t *eng) { return 0; }
 static int dummy_write (jack_driver_t *drv,
@@ -105,13 +107,16 @@ jack_driver_nt_become_real_time (jack_driver_nt_t* driver)
                 return -1;
         }
 
-        if (mlockall (MCL_CURRENT | MCL_FUTURE) != 0) {
+#ifdef USE_MLOCK
+        if (driver->engine->control->do_mlock
+	    && (mlockall (MCL_CURRENT | MCL_FUTURE) != 0)) {
 		jack_error ("cannot lock down memory for RT thread (%s)",
 			    strerror (errno));
 #ifdef ENSURE_MLOCK
 		return -1;
 #endif /* ENSURE_MLOCK */
         }
+#endif /* USE_MLOCK */
 
         return 0;
 }
